@@ -15,6 +15,7 @@
 
 #include <glib/gi18n.h>
 
+#include "gtk3-to-4.h"
 #include "chatty-purple.h"
 #include "chatty-contact.h"
 #include "chatty-contact-list.h"
@@ -40,6 +41,7 @@ struct _ChattyListRow
   GtkWidget     *add_contact_button;
   GtkWidget     *call_button;
 
+  GtkPopover    *popover;
   ChattyItem    *item;
   gboolean       hide_chat_details;
   gulong         clock_id;
@@ -133,6 +135,29 @@ list_row_user_flag_to_str (ChattyUserFlag flags)
 }
 #endif
 
+static void
+chatty_list_row_call_copy_number (GtkWidget  *widget,
+                                  const char *action_name,
+                                  GVariant   *param)
+{
+  ChattyListRow *self = CHATTY_LIST_ROW (widget);
+  GdkClipboard *clipboard;
+
+  g_assert (CHATTY_IS_LIST_ROW (self));
+
+  clipboard = gdk_display_get_clipboard (gdk_display_get_default ());
+  gdk_clipboard_set_text (clipboard, gtk_label_get_label (GTK_LABEL (self->subtitle)));
+}
+
+static void
+long_pressed (GtkGestureLongPress *gesture,
+              gdouble              x,
+              gdouble              y,
+              ChattyListRow        *self)
+{
+  gtk_popover_popup (GTK_POPOVER (self->popover));
+}
+
 static void
 chatty_list_row_update (ChattyListRow *self)
 {
@@ -155,7 +180,7 @@ chatty_list_row_update (ChattyListRow *self)
       flag = chatty_pp_buddy_get_flags (CHATTY_PP_BUDDY (self->item));
       markup = list_row_user_flag_to_str (flag);
       gtk_label_set_markup (GTK_LABEL (self->subtitle), markup);
-      gtk_widget_show (self->subtitle);
+      gtk_widget_set_visible (self->subtitle, TRUE);
     }
   } else
 #endif
@@ -172,6 +197,14 @@ chatty_list_row_update (ChattyListRow *self)
       type = g_strconcat (chatty_contact_get_value_type (CHATTY_CONTACT (self->item)), number, NULL);
     gtk_label_set_label (GTK_LABEL (self->subtitle), type);
     chatty_item_get_avatar (self->item);
+
+   if (*number) {
+      GtkGesture    *gesture;
+      gesture = gtk_gesture_long_press_new ();
+      gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (gesture));
+      g_signal_connect (gesture, "pressed", G_CALLBACK (long_pressed), self);
+   }
+
   } else if (CHATTY_IS_CHAT (self->item) && !self->hide_chat_details) {
     g_autofree char *unread = NULL;
     const char *last_message;
@@ -210,13 +243,17 @@ chatty_list_row_update (ChattyListRow *self)
     if (chatty_chat_get_chat_state (CHATTY_CHAT (self->item)) == CHATTY_CHAT_INVITED ||
         CHATTY_IS_MA_KEY_CHAT (self->item)) {
       gtk_label_set_text (GTK_LABEL (self->unread_message_count), "⚫︎");
-      gtk_widget_show (self->unread_message_count);
-      gtk_container_child_set (GTK_CONTAINER (self->content_grid), self->unread_message_count,
-                               "top-attach", 0, NULL);
+      gtk_widget_set_visible (self->unread_message_count, TRUE);
+      g_object_ref (self->unread_message_count);
+      gtk_grid_remove (GTK_GRID (self->content_grid), self->unread_message_count);
+      gtk_grid_attach (GTK_GRID (self->content_grid), self->unread_message_count, 2, 0, 1, 1);
+      g_object_unref (self->unread_message_count);
     } else {
-      gtk_container_child_set (GTK_CONTAINER (self->content_grid), self->unread_message_count,
-                               "top-attach", 1, NULL);
-      }
+      g_object_ref (self->unread_message_count);
+      gtk_grid_remove (GTK_GRID (self->content_grid), self->unread_message_count);
+      gtk_grid_attach (GTK_GRID (self->content_grid), self->unread_message_count, 2, 1, 1, 1);
+      g_object_unref (self->unread_message_count);
+    }
 
     chatty_list_row_update_last_modified (self);
   }
@@ -234,7 +271,7 @@ write_eds_contact_cb (GObject      *object,
   g_autoptr(GError) error = NULL;
 
   if (chatty_eds_write_contact_finish (result, &error)) {
-    gtk_widget_hide (self->add_contact_button);
+    gtk_widget_set_visible (self->add_contact_button, FALSE);
     return;
   }
 }
@@ -273,7 +310,6 @@ chatty_list_row_add_contact_clicked_cb (ChattyListRow *self)
 static void
 chatty_list_row_call_button_clicked_cb (ChattyListRow *self)
 {
-  g_autoptr(GError) error = NULL;
   g_autofree char *uri = NULL;
 
   g_return_if_fail (CHATTY_IS_CONTACT (self->item));
@@ -281,8 +317,7 @@ chatty_list_row_call_button_clicked_cb (ChattyListRow *self)
   uri = g_strconcat ("tel://", chatty_item_get_username (self->item), NULL);
 
   g_debug ("Calling uri: %s", uri);
-  if (!gtk_show_uri_on_window (NULL, uri, GDK_CURRENT_TIME, &error))
-    g_warning ("Failed to launch call: %s", error->message);
+  gtk_show_uri (NULL, uri, GDK_CURRENT_TIME);
 }
 
 static void
@@ -317,10 +352,13 @@ chatty_list_row_class_init (ChattyListRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, ChattyListRow, unread_message_count);
   gtk_widget_class_bind_template_child (widget_class, ChattyListRow, add_contact_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyListRow, call_button);
+  gtk_widget_class_bind_template_child (widget_class, ChattyListRow, popover);
 
   gtk_widget_class_bind_template_callback (widget_class, chatty_list_row_delete_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, chatty_list_row_add_contact_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, chatty_list_row_call_button_clicked_cb);
+
+  gtk_widget_class_install_action (widget_class, "win.copy-number", NULL, chatty_list_row_call_copy_number);
 }
 
 static void
@@ -407,8 +445,7 @@ chatty_list_row_select (ChattyListRow *self, gboolean enable)
 {
   g_return_if_fail (CHATTY_IS_LIST_ROW (self));
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->checkbox),
-                                enable);
+  gtk_check_button_set_active (GTK_CHECK_BUTTON (self->checkbox), enable);
 }
 
 void
@@ -436,7 +473,7 @@ chatty_list_row_set_call (ChattyListRow *self, gboolean enable)
 
     gtk_widget_set_visible (self->call_button, enable && user_valid);
   } else
-    gtk_widget_hide (self->call_button);
+    gtk_widget_set_visible (self->call_button, FALSE);
 }
 
 void
@@ -444,6 +481,6 @@ chatty_list_row_show_delete_button (ChattyListRow *self)
 {
   g_return_if_fail (CHATTY_IS_LIST_ROW (self));
 
-  gtk_widget_show (self->close_button);
-  gtk_widget_hide (self->checkbox);
+  gtk_widget_set_visible (self->close_button, TRUE);
+  gtk_widget_set_visible (self->checkbox, FALSE);
 }
