@@ -56,6 +56,8 @@ enum {
   PROP_SEND_RECEIPTS,
   PROP_MESSAGE_CARBONS,
   PROP_SEND_TYPING,
+  PROP_STRIP_URL_TRACKING_IDS,
+  PROP_RENDER_ATTACHMENTS,
   PROP_BLUR_IDLE_BUDDIES,
   PROP_INDICATE_UNKNOWN_CONTACTS,
   PROP_CONVERT_EMOTICONS,
@@ -98,6 +100,14 @@ chatty_settings_get_property (GObject    *object,
       g_value_set_boolean (value, chatty_settings_get_send_typing (self));
       break;
 
+    case PROP_STRIP_URL_TRACKING_IDS:
+      g_value_set_boolean (value, chatty_settings_get_strip_url_tracking_ids (self));
+      break;
+
+    case PROP_RENDER_ATTACHMENTS:
+      g_value_set_boolean (value, chatty_settings_get_render_attachments (self));
+      break;
+
     case PROP_BLUR_IDLE_BUDDIES:
       g_value_set_boolean (value, chatty_settings_get_blur_idle_buddies (self));
       break;
@@ -164,6 +174,16 @@ chatty_settings_set_property (GObject      *object,
                               g_value_get_boolean (value));
       break;
 
+    case PROP_STRIP_URL_TRACKING_IDS:
+      g_settings_set_boolean (self->settings, "strip-url-tracking-id",
+                              g_value_get_boolean (value));
+      break;
+
+    case PROP_RENDER_ATTACHMENTS:
+      g_settings_set_boolean (self->settings, "render-attachments",
+                              g_value_get_boolean (value));
+      break;
+
     case PROP_BLUR_IDLE_BUDDIES:
       g_settings_set_boolean (self->settings, "blur-idle-buddies",
                               g_value_get_boolean (value));
@@ -217,6 +237,10 @@ chatty_settings_constructed (GObject *object)
                    self, "purple-enabled", G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (self->settings, "send-typing",
                    self, "send-typing", G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (self->settings, "strip-url-tracking-id",
+                   self, "strip-url-tracking-id", G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (self->settings, "render-attachments",
+                   self, "render-attachments", G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (self->settings, "blur-idle-buddies",
                    self, "blur-idle-buddies", G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (self->settings, "indicate-unknown-contacts",
@@ -292,6 +316,20 @@ chatty_settings_class_init (ChattySettingsClass *klass)
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  properties[PROP_STRIP_URL_TRACKING_IDS] =
+    g_param_spec_boolean ("strip-url-tracking-id",
+                          "Strip Tracking IDs",
+                          "Remove Tracking IDs from URLs",
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  properties[PROP_RENDER_ATTACHMENTS] =
+    g_param_spec_boolean ("render-attachments",
+                          "Render Attachments",
+                          "Render Attachments for Messages",
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   properties[PROP_BLUR_IDLE_BUDDIES] =
     g_param_spec_boolean ("blur-idle-buddies",
                           "Blur Idle Buddies",
@@ -454,6 +492,40 @@ chatty_settings_get_send_typing (ChattySettings *self)
   return g_settings_get_boolean (self->settings, "send-typing");
 }
 
+/**
+ * chatty_settings_get_strip_url_tracking_ids:
+ * @self: A #ChattySettings
+ *
+ * Get if Chattty should automatically strip tracking IDs from URLs.
+ *
+ * Returns: %TRUE if tracking IDs are stripped automatically.
+ * %FALSE otherwise
+ */
+gboolean
+chatty_settings_get_strip_url_tracking_ids (ChattySettings *self)
+{
+  g_return_val_if_fail (CHATTY_IS_SETTINGS (self), FALSE);
+
+  return g_settings_get_boolean (self->settings, "strip-url-tracking-id");
+}
+
+/**
+ * chatty_settings_get_render_attachments:
+ * @self: A #ChattySettings
+ *
+ * Get if Chattty should render attachments from messages.
+ *
+ * Returns: %TRUE if attachments are rendered.
+ * %FALSE otherwise
+ */
+gboolean
+chatty_settings_get_render_attachments (ChattySettings *self)
+{
+  g_return_val_if_fail (CHATTY_IS_SETTINGS (self), FALSE);
+
+  return g_settings_get_boolean (self->settings, "render-attachments");
+}
+
 gboolean
 chatty_settings_get_blur_idle_buddies (ChattySettings *self)
 {
