@@ -18,6 +18,7 @@
 # include "chatty-pp-chat-info.h"
 #endif
 
+#include "gtk3-to-4.h"
 #include "chatty-mm-chat.h"
 #include "chatty-ma-chat.h"
 #include "chatty-utils.h"
@@ -55,8 +56,8 @@ info_dialog_new_invite_clicked_cb (ChattyInfoDialog *self)
 {
   g_assert (CHATTY_IS_INFO_DIALOG (self));
 
-  gtk_widget_hide (self->new_invite_button);
-  gtk_widget_show (self->invite_button);
+  gtk_widget_set_visible (self->new_invite_button, FALSE);
+  gtk_widget_set_visible (self->invite_button, TRUE);
   gtk_stack_set_visible_child (GTK_STACK (self->main_stack),
                                self->invite_page);
   gtk_widget_grab_focus (self->contact_id_entry);
@@ -67,8 +68,8 @@ info_dialog_cancel_clicked_cb (ChattyInfoDialog *self)
 {
   g_assert (CHATTY_IS_INFO_DIALOG (self));
 
-  gtk_widget_hide (self->invite_button);
-  gtk_widget_show (self->new_invite_button);
+  gtk_widget_set_visible (self->invite_button, FALSE);
+  gtk_widget_set_visible (self->new_invite_button, TRUE);
   gtk_stack_set_visible_child (GTK_STACK (self->main_stack),
                                self->chat_type_stack);
 }
@@ -77,7 +78,7 @@ static void
 cancel_button_clicked_cb (ChattyInfoDialog *self)
 {
   g_assert (CHATTY_IS_INFO_DIALOG (self));
-  gtk_widget_hide (GTK_WIDGET (self));
+  gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
 
   chatty_mm_chat_info_cancel_changes (CHATTY_MM_CHAT_INFO (self->mm_chat_info),
                                       self->chat);
@@ -88,7 +89,7 @@ apply_button_clicked_cb (ChattyInfoDialog *self)
 {
 
   g_assert (CHATTY_IS_INFO_DIALOG (self));
-  gtk_widget_hide (GTK_WIDGET (self));
+  gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
 
   chatty_mm_chat_info_apply_changes (CHATTY_MM_CHAT_INFO (self->mm_chat_info),
                                      self->chat);
@@ -112,15 +113,15 @@ info_dialog_invite_clicked_cb (ChattyInfoDialog *self)
   g_assert (CHATTY_IS_INFO_DIALOG (self));
   g_return_if_fail (self->chat);
 
-  name = gtk_entry_get_text (GTK_ENTRY (self->contact_id_entry));
-  invite_message = gtk_entry_get_text (GTK_ENTRY (self->message_entry));
+  name = gtk_editable_get_text (GTK_EDITABLE (self->contact_id_entry));
+  invite_message = gtk_editable_get_text (GTK_EDITABLE (self->message_entry));
 
   if (name && *name)
     chatty_chat_invite_async (CHATTY_CHAT (self->chat), name, invite_message, NULL,
                               chat_invite_cb, g_object_ref (self));
 
-  gtk_entry_set_text (GTK_ENTRY (self->contact_id_entry), "");
-  gtk_entry_set_text (GTK_ENTRY (self->message_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->contact_id_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->message_entry), "");
   info_dialog_cancel_clicked_cb (self);
 }
 
@@ -136,7 +137,7 @@ info_dialog_contact_id_changed_cb (ChattyInfoDialog *self,
   g_return_if_fail (self->chat);
 
   item_protocol = chatty_item_get_protocols (CHATTY_ITEM (self->chat));
-  username = gtk_entry_get_text (entry);
+  username = gtk_editable_get_text (GTK_EDITABLE (entry));
 
   protocol = chatty_utils_username_is_valid (username, item_protocol);
   gtk_widget_set_sensitive (self->invite_button, protocol == item_protocol);
@@ -197,7 +198,7 @@ chatty_info_dialog_init (ChattyInfoDialog *self)
 
 #ifdef PURPLE_ENABLED
   self->pp_chat_info = chatty_pp_chat_info_new ();
-  gtk_container_add (GTK_CONTAINER (self->chat_type_stack), self->pp_chat_info);
+  gtk_stack_add_child (GTK_STACK (self->chat_type_stack), self->pp_chat_info);
 #endif
 }
 
@@ -219,16 +220,16 @@ chatty_info_dialog_set_chat (ChattyInfoDialog *self,
   g_return_if_fail (CHATTY_IS_INFO_DIALOG (self));
   g_return_if_fail (!chat || CHATTY_IS_CHAT (chat));
 
-  gtk_entry_set_text (GTK_ENTRY (self->contact_id_entry), "");
-  gtk_entry_set_text (GTK_ENTRY (self->message_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->contact_id_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->message_entry), "");
 
   if (!g_set_object (&self->chat, chat))
     return;
 
 
-  gtk_widget_hide (self->cancel_button);
-  gtk_widget_hide (self->apply_button);
-  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), TRUE);
+  gtk_widget_set_visible (self->cancel_button, FALSE);
+  gtk_widget_set_visible (self->apply_button, FALSE);
+  gtk_header_bar_set_show_title_buttons (GTK_HEADER_BAR (self->header_bar), TRUE);
 
   if (CHATTY_IS_MA_CHAT (chat)) {
     chatty_chat_info_set_item (CHATTY_CHAT_INFO (self->ma_chat_info), chat);
@@ -242,9 +243,9 @@ chatty_info_dialog_set_chat (ChattyInfoDialog *self,
       chatty_chat_info_set_item (CHATTY_CHAT_INFO (self->pp_chat_info), NULL);
     chatty_chat_info_set_item (CHATTY_CHAT_INFO (self->mm_chat_info), chat);
     if (!chatty_chat_is_im (chat)) {
-      gtk_widget_show (self->cancel_button);
-      gtk_widget_show (self->apply_button);
-      gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header_bar), FALSE);
+      gtk_widget_set_visible (self->cancel_button, TRUE);
+      gtk_widget_set_visible (self->apply_button, TRUE);
+      gtk_header_bar_set_show_title_buttons (GTK_HEADER_BAR (self->header_bar), FALSE);
     }
     gtk_stack_set_visible_child (GTK_STACK (self->chat_type_stack), self->mm_chat_info);
   } else {
@@ -257,7 +258,7 @@ chatty_info_dialog_set_chat (ChattyInfoDialog *self,
 
   if (chatty_item_get_protocols (CHATTY_ITEM (chat)) == CHATTY_PROTOCOL_XMPP &&
       !chatty_chat_is_im (self->chat))
-    gtk_widget_show (self->new_invite_button);
+    gtk_widget_set_visible (self->new_invite_button, TRUE);
   else
-    gtk_widget_hide (self->new_invite_button);
+    gtk_widget_set_visible (self->new_invite_button, FALSE);
 }
